#
# Copyright 2023 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Package "selfservice.usage.user"
"""
from urllib.parse import urlencode
from delphixpy.v1_11_2 import response_validator

def list_user_usage_data(engine, user=None):
    """
    Lists the usage breakdown for Self-Service users.

    :param engine: The Delphix Engine
    :type engine: :py:class:`delphixpy.v1_11_2.delphix_engine.DelphixEngine`
    :param user: If passed in, this query parameter restricts the API to only
        return the usage information for the given user. By default, the usage
        information for all users is returned.
    :type user: ``str``
    :rtype: ``list`` of :py:class:`v1_11_2.web.vo.JSUserUsageData`
    """
    url = "/resources/json/delphix/selfservice/usage/user/listUserUsageData"
    query_params = {"user": user}
    query_dct = {k: query_params[k] for k in query_params if query_params[k] is not None}
    if query_dct:
        url_params = urlencode(query_dct)
        url += "?%s" % url_params
    response = engine.get(url)
    result = response_validator.validate(response, engine)
    raw_result = getattr(engine, 'raw_result', False)
    return response_validator.parse_result(result, undef_enabled=True, return_types=['JSUserUsageData'], returns_list=True, raw_result=raw_result)

